C foo_to_v5d.f

C This is a skeleton conversion program for converting your data format
C to VIS-5D's v5d format (which is directly read by vis5d).

C This version only allows a rectangular lat/lon projection and equally
C spaced vertical levels in km.  See foo2_to_v5d for more advanced
C options.

C You need to insert code to do two things:
C     1. Open your input file and read the header information, putting
C        that information into the required variables.  If your file
C        doesn't contain some of the needed values you can just assign
C        constants to the variables.
C     2. read your grid data into the array G for each timestep and
C        variable.

C Pay special attention to comments in UPPERCASE.  They give more
C information about what you have to do.

C The rest of the program will handle creating the v5d file, writing the
C grid data, and closing the file.

C Compile this program with the foo_to_v5d.f.m makefile.
C Run this program with two arguments:  the name of your input file and
C the name of the v5d output file.



      program foo_to_v5d
      implicit none
C v5df.h

C Include file for using v5d functions from FORTRAN programs


C Function prototypes.  See the README file for details.  These are
C the functions you'll want to use for writing v5d file converters.

      integer v5dcreate

      integer v5dcreatesimple

      integer v5dwrite

      integer v5dmcfile

      integer v5dclose


C 5-D grid limits, must match those in v5d.h!!!
      integer MAXVARS, MAXTIMES, MAXROWS, MAXCOLUMNS, MAXLEVELS

      parameter (MAXVARS=2)
      parameter (MAXTIMES=1)
      parameter (MAXROWS=201)
      parameter (MAXCOLUMNS=201)
      parameter (MAXLEVELS=101)

C Missing values
      real MISSING
      integer IMISSING
      integer i,j,k
      parameter (MISSING=1.0E35)
      parameter (IMISSING=-987654)

      integer iargc
c     Local vars
      integer n
      real*4 G(MAXROWS, MAXCOLUMNS, MAXLEVELS)
      real*4 elf(MAXROWS*MAXCOLUMNS*MAXLEVELS)
      character*100 inname, outname, dunname, filebas


C  THE FOLLOWING VARIABLES DESCRIBE THE DATASET TO BE CONVERTED.  YOU
C  MUST INITIALIZE ALL THESE VARIABLES WITH VALUES FROM YOUR FILE OR
C  ASSIGN SUITABLE CONSTANTS.  SEE THE README FILE FOR DESCRIPTIONS
C  OF THESE VARIABLES.
      integer nr, nc, nl
      integer numtimes
      integer numvars,step
      character*10 varname(MAXVARS)
      character*1 auxname(100)
      integer dates(MAXTIMES)
      integer times(MAXTIMES)
      integer npx,npy,npz,npts,ijk
      integer*2 code(MAXROWS*MAXCOLUMNS*MAXLEVELS)
      integer natoms, nbasin,nat
      double precision xc,yc,zc,volume
      character*16 b_name
      character*4 atom_name
      real northlat
      real latinc
      real westlon
      real loninc
      real bottomhgt
      real hgtinc
      double precision xmin,xmax,ymin,ymax,zmin,zmax
      logical if_file,file_type
      common /values/ elf,step,npx,npy,npz,if_file,code
      equivalence (auxname(1),dunname)
c     initialize the variables to missing values
      data nr,nc,nl / IMISSING, IMISSING, IMISSING /
      data numtimes,numvars / 1, IMISSING /
      data (varname(i),i=1,MAXVARS) / "ELF       ","BAS       " /
      data (dates(i),i=1,MAXTIMES) / 1 /
      data (times(i),i=1,MAXTIMES) / 1/
      data northlat, latinc / MISSING, MISSING /
      data westlon, loninc / MISSING, MISSING /
      data bottomhgt, hgtinc / MISSING, MISSING /

c     get command line arguments
      file_type=.true.
      if (iargc() .ne. 2) then
         print *,"Error:  two filename arguments are needed."
         call exit(1)
      else
         call getarg(1, inname)
         call getarg(2, outname)
      endif
      print 10,"Input file: ", inname
      print 10,"Output file: ", outname
      dunname=inname
      numvars=1
      i=1
      do while(auxname(i).ne.'_')
         i=i+1
      enddo
      write(*,*)  " type of file syn/bas "
      read(*,'(3a1)') (auxname(i+k),k=1,3)
      if(auxname(i+1).eq.'s') file_type=.false.
      filebas=dunname
 10   format (A,A)


C  OPEN YOUR DATAFILE (inname) HERE AND READ ITS HEADER INFORMATION TO
C  INITIALIZE THE ABOVE VARIABLES.
      open(unit=1,file=inname,status='unknown',form='unformatted')
      read(1) npx,npy,npz
      read(1) xmin,xmax,ymin,ymax,zmin,zmax
      write(*,'(3i5)') npx,npy,npz
      write(*,'(6f12.6)') xmin,xmax,ymin,ymax,zmin,zmax
      write(*,*) 'step '
      read(*,*) step 
      write(*,*) ' command line '
      write(*,'(2x,"vis5d -box ",3f8.3,2x,a100)')
     .              ymax-ymin,xmax-xmin,zmax-zmin,outname

      nr=npx/step
      nc=npy/step
      nl=npz/step
      npts=npx*npy*npz
      northlat=xmin
      latinc=(xmax-xmin)/(npx-1)
      westlon=ymin
      loninc=(ymax-ymin)/(npy-1)
      bottomhgt=zmin
      hgtinc=(zmax-zmin)/(npz-1)
      read(1)(elf(i),i=1,npts)
      inquire(file=filebas,exist=if_file)
      if(if_file) then
         open(unit=2,file=filebas,status='old',form='unformatted')
         read(2) npx,npy,npz
         read(2) xmin,xmax,ymin,ymax,zmin,zmax
         if(file_type) then
            read(2) natoms,nbasin
            do i=1,natoms
               read(2) nat,atom_name,xc,yc,zc
            enddo
            do i=1,nbasin
               read(2) nat,b_name,xc,yc,zc,volume
            enddo
         endif
         read(2)(code(i),i=1,npts)
         close(unit=2)
         numvars=2
      endif




C  IF YOU FAIL TO INITIALIZE SOME VARIABLES, THEY WILL BE DETECTED AND
C  REPORTED BY THE V5D LIBRARY.

c     Create the v5d file.
      n = v5dcreatesimple( outname, numtimes, numvars, nr, nc, nl,
     *                 varname, times, dates,
     *                 northlat, latinc, westlon, loninc,
     *                 bottomhgt, hgtinc )
      if (n .eq. 0) then
         call exit(1)
      endif

c     enter the conversion subroutine
      call convert( nr, nc, nl, numtimes, numvars, G )

c     close the v5d file and exit
      n = v5dclose()
      if (n .eq. 0) then
c        failed
         call exit(1)
      else
c        success
         call exit(0)
      endif

      end




c     Read 3-D grids from input file, write them to the output file.
      subroutine convert( nr, nc, nl, numtimes, numvars, G )
      implicit none
      include "v5df.h"
c     Arguments
      integer nr
      integer nc
      integer nl
      integer i,j,k,ix,iy,iz,npx,npy,npz
      integer numtimes
      integer numvars,ijk,step
      real*4 elf(4080501),G(nr, nc, nl)
      integer*2 code(4080501)
c     Local vars
      integer it, iv, n
      logical if_file
      common /values/elf,step,npx,npy,npz,if_file,code
      it=1
      iv=1
      i=0
      do ix=1,npx,step
         i=i+1
         j=0
         do iy=1,npy,step
            j=j+1
            k=0
            do iz=1,npz,step
               k=k+1
               ijk=1+iz+npz*(iy+npy*ix)
               g(i,j,k)=elf(ijk)
            enddo
         enddo
      enddo
c       write the 3-D grid to the v5d file
      n = v5dwrite( IT, IV, G )
      if (n .eq. 0) then
c          error
         call exit(1)
      endif
      if(if_file) then
        iv=2
        i=0
        do ix=1,npx,step
           i=i+1
           j=0
           do iy=1,npy,step
              j=j+1
              k=0
              do iz=1,npz,step
                 k=k+1
                 ijk=1+iz+npz*(iy+npy*ix)
                 g(i,j,k)=code(ijk)
              enddo
           enddo
        enddo
c       write the 3-D grid to the v5d file
        n = v5dwrite( IT, IV, G )
        if (n .eq. 0) then
c          error
           call exit(1)
        endif
      endif
      end
